try:
    from deep_utils.dummy_objects.utils.memory_utils import MemoryUtilsTorch
    from deep_utils.utils.memory_utils.torch_memory_utils import MemoryUtilsTorch
except ModuleNotFoundError:
    pass
