from .cutmix_tf import CutMixTF

try:
    from deep_utils.augmentation.cutmix.cutmix_torch import CutMixTorch
    from deep_utils.dummy_objects.augmentation.cutmix import CutMixTorch
except:
    pass
