from deep_utils.utils.lib_utils import import_module

SSDCV2CaffeFaceDetector = import_module(
    "deep_utils.vision.face_detection.ssd.cv2.caffe.ssd_cv2_caffe_face_detection",
    "SSDCV2CaffeFaceDetector",
)
