try:
    from deep_utils.dummy_objects.utils.sqlalchemy_utils import SQLAlchemyChecks
    from .checks import SQLAlchemyChecks
except ModuleNotFoundError:
    pass

try:
    from deep_utils.dummy_objects.utils.sqlalchemy_utils import SQLAlchemyInserts
    from .insert import SQLAlchemyInserts
except ModuleNotFoundError:
    pass

try:
    from deep_utils.dummy_objects.utils.sqlalchemy_utils import SQLAlchemyUtils
    from .sqlalchemy_utils import SQLAlchemyUtils
except ModuleNotFoundError:
    pass
